@@ -1719,7 +1719,6 @@ gboolean
 dfu_device_attach (DfuDevice *device, GError **error)
 {
 	DfuDevicePrivate *priv = GET_PRIVATE (device);
-	g_autoptr(GError) error_local = NULL;
 
 	g_return_val_if_fail (DFU_IS_DEVICE (device), FALSE);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
@@ -1834,7 +1833,7 @@ dfu_device_upload (DfuDevice *device,
 
 	/* ensure interface is claimed */
 	if (!dfu_device_ensure_interface (device, cancellable, error))
-		return FALSE;
+		return NULL;
 
 	/* create ahead of time */
 	firmware = dfu_firmware_new ();
@@ -1974,7 +1973,6 @@ dfu_device_download (DfuDevice *device,
 	GPtrArray *images;
 	gboolean ret;
 	guint i;
-	g_autoptr(GPtrArray) targets = NULL;
 
 	/* no backing USB device */
 	if (priv->dev == NULL) {
@@ -2011,33 +2009,37 @@ dfu_device_download (DfuDevice *device,
 	}
 
 	/* check vendor matches */
-	if (!dfu_device_id_compatible (dfu_firmware_get_vid (firmware),
-				       priv->runtime_vid,
-				       g_usb_device_get_vid (priv->dev))) {
-		g_set_error (error,
-			     DFU_ERROR,
-			     DFU_ERROR_NOT_SUPPORTED,
-			     "vendor ID incorrect, expected 0x%04x "
-			     "got 0x%04x and 0x%04x\n",
-			     dfu_firmware_get_vid (firmware),
-			     priv->runtime_vid,
-			     g_usb_device_get_vid (priv->dev));
-		return FALSE;
+	if (priv->runtime_vid != 0xffff) {
+		if (!dfu_device_id_compatible (dfu_firmware_get_vid (firmware),
+					       priv->runtime_vid,
+					       g_usb_device_get_vid (priv->dev))) {
+			g_set_error (error,
+				     DFU_ERROR,
+				     DFU_ERROR_NOT_SUPPORTED,
+				     "vendor ID incorrect, expected 0x%04x "
+				     "got 0x%04x and 0x%04x\n",
+				     dfu_firmware_get_vid (firmware),
+				     priv->runtime_vid,
+				     g_usb_device_get_vid (priv->dev));
+			return FALSE;
+		}
 	}
 
 	/* check product matches */
-	if (!dfu_device_id_compatible (dfu_firmware_get_pid (firmware),
-				       priv->runtime_pid,
-				       g_usb_device_get_pid (priv->dev))) {
-		g_set_error (error,
-			     DFU_ERROR,
-			     DFU_ERROR_NOT_SUPPORTED,
-			     "product ID incorrect, expected 0x%04x "
-			     "got 0x%04x and 0x%04x",
-			     dfu_firmware_get_pid (firmware),
-			     priv->runtime_pid,
-			     g_usb_device_get_pid (priv->dev));
-		return FALSE;
+	if (priv->runtime_pid != 0xffff) {
+		if (!dfu_device_id_compatible (dfu_firmware_get_pid (firmware),
+					       priv->runtime_pid,
+					       g_usb_device_get_pid (priv->dev))) {
+			g_set_error (error,
+				     DFU_ERROR,
+				     DFU_ERROR_NOT_SUPPORTED,
+				     "product ID incorrect, expected 0x%04x "
+				     "got 0x%04x and 0x%04x",
+				     dfu_firmware_get_pid (firmware),
+				     priv->runtime_pid,
+				     g_usb_device_get_pid (priv->dev));
+			return FALSE;
+		}
 	}
 
 	/* APP -> DFU */
