@@ -74,13 +74,10 @@ _get_element_from_section_name (Elf *e, const gchar *desired_name)
 	const gchar *name;
 	size_t shstrndx;
 
-	if (elf_getshdrstrndx (e, &shstrndx) != 0) {
-		g_warning ("failed elf_getshdrstrndx");
+	if (elf_getshdrstrndx (e, &shstrndx) != 0)
 		return NULL;
-	}
 	while ((scn = elf_nextscn (e, scn)) != NULL ) {
 		if (gelf_getshdr (scn, &shdr ) != & shdr) {
-			g_warning ("failed gelf_getshdr");
 			continue;
 		}
 
@@ -90,7 +87,6 @@ _get_element_from_section_name (Elf *e, const gchar *desired_name)
 
 		/* not the same section name */
 		if ((name = elf_strptr (e, shstrndx, shdr.sh_name)) == NULL) {
-			g_warning ("failed elf_strptr");
 			continue;
 		}
 		if (g_strcmp0 (name, desired_name) == 0) {
@@ -116,19 +112,15 @@ dfu_format_elf_symbols_from_symtab (DfuFirmware *firmware, Elf *e)
 	Elf_Scn *scn = NULL;
 	gsize shstrndx;
 
-	if (elf_getshdrstrndx (e, &shstrndx) != 0) {
-		g_warning ("failed elf_getshdrstrndx");
+	if (elf_getshdrstrndx (e, &shstrndx) != 0)
 		return;
-	}
 	while ((scn = elf_nextscn (e, scn)) != NULL ) {
 		Elf_Data *data;
 		GElf_Shdr shdr;
 		const gchar *name;
 		gssize ns;
-		if (gelf_getshdr (scn, &shdr) != &shdr) {
-			g_warning ("failed gelf_getshdr");
+		if (gelf_getshdr (scn, &shdr) != &shdr)
 			continue;
-		}
 
 		/* not program data */
 		if (shdr.sh_type != SHT_SYMTAB)
@@ -350,7 +342,7 @@ dfu_firmware_to_elf (DfuFirmware *firmware, GError **error)
 				     DFU_ERROR,
 				     DFU_ERROR_INTERNAL,
 				     "no image to write");
-		return FALSE;
+		return NULL;
 	}
 
 	/* load library */
@@ -360,7 +352,7 @@ dfu_firmware_to_elf (DfuFirmware *firmware, GError **error)
 			     DFU_ERROR_INTERNAL,
 			     "ELF library init failed: %s",
 			     elf_errmsg (-1));
-		return FALSE;
+		return NULL;
 	}
 
 	/* create from buffer */
@@ -386,7 +378,10 @@ dfu_firmware_to_elf (DfuFirmware *firmware, GError **error)
 	/* add executable header */
 	ehdr = elf32_newehdr (e);
 	if (ehdr == NULL) {
-		g_warning ("failed to create executable header: %s",
+		g_set_error (error,
+			     DFU_ERROR,
+			     DFU_ERROR_INTERNAL,
+			     "failed to create executable header: %s",
 			     elf_errmsg (-1));
 		return NULL;
 	}
@@ -396,7 +391,7 @@ dfu_firmware_to_elf (DfuFirmware *firmware, GError **error)
 
 	/* pack the image */
 	if (!dfu_format_elf_pack_image (e, image, error))
-		return FALSE;
+		return NULL;
 
 	/* allocate section for holding the string table */
 	scn = elf_newscn (e);
@@ -468,6 +463,6 @@ dfu_firmware_to_elf (DfuFirmware *firmware, GError **error)
 			     DFU_ERROR,
 			     DFU_ERROR_INTERNAL,
 			     "compiled without libelf support");
-	return FALSE;
+	return NULL;
 #endif
 }
