@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2016 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2016-2017 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -31,8 +31,12 @@
 #include <sys/types.h>
 
 #include "fwupd-client.h"
+#include "fwupd-common.h"
 #include "fwupd-enums.h"
 #include "fwupd-error.h"
+#include "fwupd-device-private.h"
+#include "fwupd-release-private.h"
+#include "fwupd-remote-private.h"
 #include "fwupd-result.h"
 
 static void fwupd_client_finalize	 (GObject *object);
@@ -162,7 +166,7 @@ fwupd_client_signal_cb (GDBusProxy *proxy,
 	if (g_strcmp0 (signal_name, "DeviceAdded") == 0) {
 		res = fwupd_result_new_from_data (parameters);
 		g_debug ("Emitting ::device-added(%s)",
-			 fwupd_result_get_device_id (res));
+			 fwupd_device_get_id (fwupd_result_get_device (res)));
 		g_signal_emit (client, signals[SIGNAL_DEVICE_ADDED], 0, res);
 		return;
 	}
@@ -170,18 +174,17 @@ fwupd_client_signal_cb (GDBusProxy *proxy,
 		res = fwupd_result_new_from_data (parameters);
 		g_signal_emit (client, signals[SIGNAL_DEVICE_REMOVED], 0, res);
 		g_debug ("Emitting ::device-removed(%s)",
-			 fwupd_result_get_device_id (res));
+			 fwupd_device_get_id (fwupd_result_get_device (res)));
 		return;
 	}
 	if (g_strcmp0 (signal_name, "DeviceChanged") == 0) {
 		res = fwupd_result_new_from_data (parameters);
 		g_signal_emit (client, signals[SIGNAL_DEVICE_CHANGED], 0, res);
 		g_debug ("Emitting ::device-changed(%s)",
-			 fwupd_result_get_device_id (res));
+			 fwupd_device_get_id (fwupd_result_get_device (res)));
 		return;
 	}
-	g_warning ("Unknown signal name '%s' from %s",
-		   signal_name, sender_name);
+	g_debug ("Unknown signal name '%s' from %s", signal_name, sender_name);
 }
 
 /**
@@ -214,8 +217,10 @@ fwupd_client_connect (FwupdClient *client, GCancellable *cancellable, GError **e
 
 	/* connect to the daemon */
 	priv->conn = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, error);
-	if (priv->conn == NULL)
+	if (priv->conn == NULL) {
+		g_prefix_error (error, "Failed to connect to system D-Bus: ");
 		return FALSE;
+	}
 	priv->proxy = g_dbus_proxy_new_sync (priv->conn,
 					     G_DBUS_PROXY_FLAGS_NONE,
 					     NULL,
@@ -258,6 +263,70 @@ fwupd_client_parse_results_from_data (GVariant *devices)
 	return results;
 }
 
+static GPtrArray *
+fwupd_client_parse_releases_from_variant (GVariant *val)
+{
+	GPtrArray *array = NULL;
+	gsize sz;
+	g_autoptr(GVariant) untuple = NULL;
+
+	array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+	untuple = g_variant_get_child_value (val, 0);
+	sz = g_variant_n_children (untuple);
+	for (guint i = 0; i < sz; i++) {
+		FwupdRelease *rel;
+		g_autoptr(GVariant) data = NULL;
+		data = g_variant_get_child_value (untuple, i);
+		rel = fwupd_release_new_from_data (data);
+		if (rel == NULL)
+			continue;
+		g_ptr_array_add (array, rel);
+	}
+	return array;
+}
+
+static GPtrArray *
+fwupd_client_parse_devices_from_variant (GVariant *val)
+{
+	GPtrArray *array = NULL;
+	gsize sz;
+	g_autoptr(GVariant) untuple = NULL;
+
+	array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+	untuple = g_variant_get_child_value (val, 0);
+	sz = g_variant_n_children (untuple);
+	for (guint i = 0; i < sz; i++) {
+		FwupdDevice *dev;
+		g_autoptr(GVariant) data = NULL;
+		data = g_variant_get_child_value (untuple, i);
+		dev = fwupd_device_new_from_data (data);
+		if (dev == NULL)
+			continue;
+		g_ptr_array_add (array, dev);
+	}
+	return array;
+}
+
+static GPtrArray *
+fwupd_client_parse_remotes_from_data (GVariant *devices)
+{
+	GPtrArray *remotes = NULL;
+	gsize sz;
+	g_autoptr(GVariant) untuple = NULL;
+
+	remotes = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+	untuple = g_variant_get_child_value (devices, 0);
+	sz = g_variant_n_children (untuple);
+	for (guint i = 0; i < sz; i++) {
+		FwupdRemote *remote;
+		g_autoptr(GVariant) data = g_variant_get_child_value (untuple, i);
+		remote = fwupd_remote_new_from_data (data);
+		g_ptr_array_add (remotes, remote);
+	}
+
+	return remotes;
+}
+
 static void
 fwupd_client_fixup_dbus_error (GError *error)
 {
@@ -328,6 +397,92 @@ fwupd_client_get_devices (FwupdClient *client, GCancellable *cancellable, GError
 	return fwupd_client_parse_results_from_data (val);
 }
 
+/**
+ * fwupd_client_get_devices_simple:
+ * @client: A #FwupdClient
+ * @cancellable: the #GCancellable, or %NULL
+ * @error: the #GError, or %NULL
+ *
+ * Gets all the devices registered with the daemon.
+ *
+ * Returns: (element-type FwupdDevice) (transfer container): results
+ *
+ * Since: 0.9.2
+ **/
+GPtrArray *
+fwupd_client_get_devices_simple (FwupdClient *client, GCancellable *cancellable, GError **error)
+{
+	FwupdClientPrivate *priv = GET_PRIVATE (client);
+	g_autoptr(GVariant) val = NULL;
+
+	g_return_val_if_fail (FWUPD_IS_CLIENT (client), NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* connect */
+	if (!fwupd_client_connect (client, cancellable, error))
+		return NULL;
+
+	/* call into daemon */
+	val = g_dbus_proxy_call_sync (priv->proxy,
+				      "GetDevices",
+				      NULL,
+				      G_DBUS_CALL_FLAGS_NONE,
+				      -1,
+				      cancellable,
+				      error);
+	if (val == NULL) {
+		if (error != NULL)
+			fwupd_client_fixup_dbus_error (*error);
+		return NULL;
+	}
+	return fwupd_client_parse_devices_from_variant (val);
+}
+
+/**
+ * fwupd_client_get_device_by_id:
+ * @client: A #FwupdClient
+ * @device_id: the device ID, e.g. "usb:00:01:03:03"
+ * @cancellable: the #GCancellable, or %NULL
+ * @error: the #GError, or %NULL
+ *
+ * Gets a device by it's device ID.
+ *
+ * Returns: (transfer full): a #FwupdDevice or %NULL
+ *
+ * Since: 0.9.3
+ **/
+FwupdDevice *
+fwupd_client_get_device_by_id (FwupdClient *client,
+			       const gchar *device_id,
+			       GCancellable *cancellable,
+			       GError **error)
+{
+	g_autoptr(GPtrArray) devices = NULL;
+
+	g_return_val_if_fail (FWUPD_IS_CLIENT (client), NULL);
+	g_return_val_if_fail (device_id != NULL, NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* get all the devices */
+	devices = fwupd_client_get_devices_simple (client, cancellable, error);
+	if (devices == NULL)
+		return NULL;
+
+	/* find the device by ID (client side) */
+	for (guint i = 0; i < devices->len; i++) {
+		FwupdDevice *dev = g_ptr_array_index (devices, i);
+		if (g_strcmp0 (fwupd_device_get_id (dev), device_id) == 0)
+			return g_object_ref (dev);
+	}
+	g_set_error (error,
+		     FWUPD_ERROR,
+		     FWUPD_ERROR_NOT_FOUND,
+		     "failed to find %s", device_id);
+	return NULL;
+}
+
 /**
  * fwupd_client_get_updates:
  * @client: A #FwupdClient
@@ -370,6 +525,51 @@ fwupd_client_get_updates (FwupdClient *client, GCancellable *cancellable, GError
 	return fwupd_client_parse_results_from_data (val);
 }
 
+/**
+ * fwupd_client_get_releases:
+ * @client: A #FwupdClient
+ * @device_id: the device ID
+ * @cancellable: the #GCancellable, or %NULL
+ * @error: the #GError, or %NULL
+ *
+ * Gets all the releases for a specific device
+ *
+ * Returns: (element-type FwupdRelease) (transfer container): results
+ *
+ * Since: 0.9.3
+ **/
+GPtrArray *
+fwupd_client_get_releases (FwupdClient *client, const gchar *device_id,
+			   GCancellable *cancellable, GError **error)
+{
+	FwupdClientPrivate *priv = GET_PRIVATE (client);
+	g_autoptr(GVariant) val = NULL;
+
+	g_return_val_if_fail (FWUPD_IS_CLIENT (client), NULL);
+	g_return_val_if_fail (device_id != NULL, NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* connect */
+	if (!fwupd_client_connect (client, cancellable, error))
+		return NULL;
+
+	/* call into daemon */
+	val = g_dbus_proxy_call_sync (priv->proxy,
+				      "GetReleases",
+				      g_variant_new ("(s)", device_id),
+				      G_DBUS_CALL_FLAGS_NONE,
+				      -1,
+				      cancellable,
+				      error);
+	if (val == NULL) {
+		if (error != NULL)
+			fwupd_client_fixup_dbus_error (*error);
+		return NULL;
+	}
+	return fwupd_client_parse_releases_from_variant (val);
+}
+
 static void
 fwupd_client_proxy_call_cb (GObject *source, GAsyncResult *res, gpointer user_data)
 {
@@ -402,7 +602,6 @@ fwupd_client_verify (FwupdClient *client, const gchar *device_id,
 {
 	FwupdClientPrivate *priv = GET_PRIVATE (client);
 	g_autoptr(FwupdClientHelper) helper = NULL;
-	g_autoptr(GVariant) val = NULL;
 
 	g_return_val_if_fail (FWUPD_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (device_id != NULL, FALSE);
@@ -451,7 +650,6 @@ fwupd_client_verify_update (FwupdClient *client, const gchar *device_id,
 {
 	FwupdClientPrivate *priv = GET_PRIVATE (client);
 	g_autoptr(FwupdClientHelper) helper = NULL;
-	g_autoptr(GVariant) val = NULL;
 
 	g_return_val_if_fail (FWUPD_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (device_id != NULL, FALSE);
@@ -500,7 +698,6 @@ fwupd_client_unlock (FwupdClient *client, const gchar *device_id,
 {
 	FwupdClientPrivate *priv = GET_PRIVATE (client);
 	g_autoptr(FwupdClientHelper) helper = NULL;
-	g_autoptr(GVariant) val = NULL;
 
 	g_return_val_if_fail (FWUPD_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (device_id != NULL, FALSE);
@@ -549,7 +746,6 @@ fwupd_client_clear_results (FwupdClient *client, const gchar *device_id,
 {
 	FwupdClientPrivate *priv = GET_PRIVATE (client);
 	g_autoptr(FwupdClientHelper) helper = NULL;
-	g_autoptr(GVariant) val = NULL;
 
 	g_return_val_if_fail (FWUPD_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (device_id != NULL, FALSE);
@@ -598,7 +794,6 @@ fwupd_client_get_results (FwupdClient *client, const gchar *device_id,
 {
 	FwupdClientPrivate *priv = GET_PRIVATE (client);
 	g_autoptr(FwupdClientHelper) helper = NULL;
-	g_autoptr(GVariant) val = NULL;
 
 	g_return_val_if_fail (FWUPD_IS_CLIENT (client), NULL);
 	g_return_val_if_fail (device_id != NULL, NULL);
@@ -678,7 +873,6 @@ fwupd_client_install (FwupdClient *client,
 	g_autoptr(FwupdClientHelper) helper = NULL;
 	g_autoptr(GDBusMessage) request = NULL;
 	g_autoptr(GUnixFDList) fd_list = NULL;
-	g_autoptr(GVariant) val = NULL;
 
 	g_return_val_if_fail (FWUPD_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (device_id != NULL, FALSE);
@@ -775,26 +969,11 @@ FwupdResult *
 fwupd_client_get_details (FwupdClient *client, const gchar *filename,
 			  GCancellable *cancellable, GError **error)
 {
-	FwupdClientPrivate *priv = GET_PRIVATE (client);
-	GVariant *body;
-	gint fd;
-	gint retval;
-	g_autoptr(FwupdClientHelper) helper = NULL;
-	g_autoptr(GDBusMessage) request = NULL;
-	g_autoptr(GUnixFDList) fd_list = NULL;
-
-	g_return_val_if_fail (FWUPD_IS_CLIENT (client), NULL);
-	g_return_val_if_fail (filename != NULL, NULL);
-	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
-	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
-
-	/* connect */
-	if (!fwupd_client_connect (client, cancellable, error))
+	g_autoptr(GPtrArray) results = NULL;
+	results = fwupd_client_get_details_local (client, filename, cancellable, error);
+	if (results == NULL)
 		return NULL;
-
-	/* open file */
-	fd = open (filename, O_RDONLY);
-	if (fd < 0) {
+	if (results->len == 0) {
 		g_set_error (error,
 			     FWUPD_ERROR,
 			     FWUPD_ERROR_INVALID_FILE,
@@ -802,42 +981,7 @@ fwupd_client_get_details (FwupdClient *client, const gchar *filename,
 			     filename);
 		return NULL;
 	}
-
-	/* set out of band file descriptor */
-	fd_list = g_unix_fd_list_new ();
-	retval = g_unix_fd_list_append (fd_list, fd, NULL);
-	g_assert (retval != -1);
-	request = g_dbus_message_new_method_call (FWUPD_DBUS_SERVICE,
-						  FWUPD_DBUS_PATH,
-						  FWUPD_DBUS_INTERFACE,
-						  "GetDetails");
-	g_dbus_message_set_unix_fd_list (request, fd_list);
-
-	/* g_unix_fd_list_append did a dup() already */
-	close (fd);
-
-	/* call into daemon */
-	helper = fwupd_client_helper_new ();
-	body = g_variant_new ("(h)", fd);
-	g_dbus_message_set_body (request, body);
-
-	g_dbus_connection_send_message_with_reply (priv->conn,
-						   request,
-						   G_DBUS_SEND_MESSAGE_FLAGS_NONE,
-						   -1,
-						   NULL,
-						   cancellable,
-						   fwupd_client_send_message_cb,
-						   helper);
-	g_main_loop_run (helper->loop);
-	if (!helper->ret) {
-		g_propagate_error (error, helper->error);
-		helper->error = NULL;
-		return NULL;
-	}
-
-	/* print results */
-	return fwupd_result_new_from_data (helper->val);
+	return g_object_ref (g_ptr_array_index (results, 0));
 }
 
 /**
@@ -998,6 +1142,42 @@ fwupd_client_update_metadata (FwupdClient *client,
 			      const gchar *signature_fn,
 			      GCancellable *cancellable,
 			      GError **error)
+{
+	return fwupd_client_update_metadata_with_id (client,
+						     "lvfs", /* remote_id */
+						     metadata_fn,
+						     signature_fn,
+						     cancellable,
+						     error);
+}
+
+/**
+ * fwupd_client_update_metadata_with_id:
+ * @client: A #FwupdClient
+ * @remote_id: the remote ID, e.g. "lvfs-testing"
+ * @metadata_fn: the XML metadata filename
+ * @signature_fn: the GPG signature file
+ * @cancellable: the #GCancellable, or %NULL
+ * @error: the #GError, or %NULL
+ *
+ * Updates the metadata. This allows a session process to download the metadata
+ * and metadata signing file to be passed into the daemon to be checked and
+ * parsed.
+ *
+ * The @remote_id allows the firmware to be tagged so that the remote can be
+ * matched when the firmware is downloaded.
+ *
+ * Returns: %TRUE for success
+ *
+ * Since: 0.9.3
+ **/
+gboolean
+fwupd_client_update_metadata_with_id (FwupdClient *client,
+				      const gchar *remote_id,
+				      const gchar *metadata_fn,
+				      const gchar *signature_fn,
+				      GCancellable *cancellable,
+				      GError **error)
 {
 	FwupdClientPrivate *priv = GET_PRIVATE (client);
 	GVariant *body;
@@ -1008,6 +1188,7 @@ fwupd_client_update_metadata (FwupdClient *client,
 	g_autoptr(GUnixFDList) fd_list = NULL;
 
 	g_return_val_if_fail (FWUPD_IS_CLIENT (client), FALSE);
+	g_return_val_if_fail (remote_id != NULL, FALSE);
 	g_return_val_if_fail (metadata_fn != NULL, FALSE);
 	g_return_val_if_fail (signature_fn != NULL, FALSE);
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
@@ -1045,7 +1226,7 @@ fwupd_client_update_metadata (FwupdClient *client,
 	request = g_dbus_message_new_method_call (FWUPD_DBUS_SERVICE,
 						  FWUPD_DBUS_PATH,
 						  FWUPD_DBUS_INTERFACE,
-						  "UpdateMetadata");
+						  "UpdateMetadataWithId");
 	g_dbus_message_set_unix_fd_list (request, fd_list);
 
 	/* g_unix_fd_list_append did a dup() already */
@@ -1053,7 +1234,7 @@ fwupd_client_update_metadata (FwupdClient *client,
 	close (fd_sig);
 
 	/* call into daemon */
-	body = g_variant_new ("(hh)", fd, fd_sig);
+	body = g_variant_new ("(shh)", remote_id, fd, fd_sig);
 	g_dbus_message_set_body (request, body);
 	helper = fwupd_client_helper_new ();
 	g_dbus_connection_send_message_with_reply (priv->conn,
@@ -1073,6 +1254,104 @@ fwupd_client_update_metadata (FwupdClient *client,
 	return TRUE;
 }
 
+/**
+ * fwupd_client_get_remotes:
+ * @client: A #FwupdClient
+ * @cancellable: the #GCancellable, or %NULL
+ * @error: the #GError, or %NULL
+ *
+ * Gets the list of remotes that have been configured for the system.
+ *
+ * Returns: (element-type FwupdRemote) (transfer container): list of remotes, or %NULL
+ *
+ * Since: 0.9.3
+ **/
+GPtrArray *
+fwupd_client_get_remotes (FwupdClient *client, GCancellable *cancellable, GError **error)
+{
+	FwupdClientPrivate *priv = GET_PRIVATE (client);
+	g_autoptr(GVariant) val = NULL;
+
+	g_return_val_if_fail (FWUPD_IS_CLIENT (client), NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* connect */
+	if (!fwupd_client_connect (client, cancellable, error))
+		return NULL;
+
+	/* call into daemon */
+	val = g_dbus_proxy_call_sync (priv->proxy,
+				      "GetRemotes",
+				      NULL,
+				      G_DBUS_CALL_FLAGS_NONE,
+				      -1,
+				      cancellable,
+				      error);
+	if (val == NULL) {
+		if (error != NULL)
+			fwupd_client_fixup_dbus_error (*error);
+		return NULL;
+	}
+	return fwupd_client_parse_remotes_from_data (val);
+}
+
+static FwupdRemote *
+fwupd_client_get_remote_by_id_noref (GPtrArray *remotes, const gchar *remote_id)
+{
+	for (guint i = 0; i < remotes->len; i++) {
+		FwupdRemote *remote = g_ptr_array_index (remotes, i);
+		if (g_strcmp0 (remote_id, fwupd_remote_get_id (remote)) == 0)
+			return remote;
+	}
+	return NULL;
+}
+
+/**
+ * fwupd_client_get_remote_by_id:
+ * @client: A #FwupdClient
+ * @remote_id: the remote ID, e.g. "lvfs-testing"
+ * @cancellable: the #GCancellable, or %NULL
+ * @error: the #GError, or %NULL
+ *
+ * Gets a specific remote that has been configured for the system.
+ *
+ * Returns: (transfer full): a #FwupdRemote, or %NULL if not found
+ *
+ * Since: 0.9.3
+ **/
+FwupdRemote *
+fwupd_client_get_remote_by_id (FwupdClient *client,
+			       const gchar *remote_id,
+			       GCancellable *cancellable,
+			       GError **error)
+{
+	FwupdRemote *remote;
+	g_autoptr(GPtrArray) remotes = NULL;
+
+	g_return_val_if_fail (FWUPD_IS_CLIENT (client), NULL);
+	g_return_val_if_fail (remote_id != NULL, NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* find remote in list */
+	remotes = fwupd_client_get_remotes (client, cancellable, error);
+	if (remotes == NULL)
+		return NULL;
+	remote = fwupd_client_get_remote_by_id_noref (remotes, remote_id);
+	if (remote == NULL) {
+		g_set_error (error,
+			     FWUPD_ERROR,
+			     FWUPD_ERROR_NOT_FOUND,
+			     "No remote '%s' found in search paths",
+			     remote_id);
+		return NULL;
+	}
+
+	/* success */
+	return g_object_ref (remote);
+}
+
 static void
 fwupd_client_get_property (GObject *object, guint prop_id,
 			   GValue *value, GParamSpec *pspec)
