@@ -52,6 +52,8 @@ fwupd_status_to_string (FwupdStatus status)
 		return "device-verify";
 	if (status == FWUPD_STATUS_SCHEDULING)
 		return "scheduling";
+	if (status == FWUPD_STATUS_DOWNLOADING)
+		return "downloading";
 	return NULL;
 }
 
@@ -84,6 +86,8 @@ fwupd_status_from_string (const gchar *status)
 		return FWUPD_STATUS_DEVICE_VERIFY;
 	if (g_strcmp0 (status, "scheduling") == 0)
 		return FWUPD_STATUS_SCHEDULING;
+	if (g_strcmp0 (status, "downloading") == 0)
+		return FWUPD_STATUS_DOWNLOADING;
 	return FWUPD_STATUS_LAST;
 }
 
@@ -104,10 +108,10 @@ fwupd_device_flag_to_string (FwupdDeviceFlags device_flag)
 		return "none";
 	if (device_flag == FWUPD_DEVICE_FLAG_INTERNAL)
 		return "internal";
-	if (device_flag == FWUPD_DEVICE_FLAG_ALLOW_ONLINE)
-		return "allow-online";
-	if (device_flag == FWUPD_DEVICE_FLAG_ALLOW_OFFLINE)
-		return "allow-offline";
+	if (device_flag == FWUPD_DEVICE_FLAG_UPDATABLE)
+		return "updatable";
+	if (device_flag == FWUPD_DEVICE_FLAG_ONLY_OFFLINE)
+		return "only-offline";
 	if (device_flag == FWUPD_DEVICE_FLAG_REQUIRE_AC)
 		return "require-ac";
 	if (device_flag == FWUPD_DEVICE_FLAG_LOCKED)
@@ -116,6 +120,10 @@ fwupd_device_flag_to_string (FwupdDeviceFlags device_flag)
 		return "supported";
 	if (device_flag == FWUPD_DEVICE_FLAG_NEEDS_BOOTLOADER)
 		return "needs-bootloader";
+	if (device_flag == FWUPD_DEVICE_FLAG_REGISTERED)
+		return "registered";
+	if (device_flag == FWUPD_DEVICE_FLAG_NEEDS_REBOOT)
+		return "needs-reboot";
 	if (device_flag == FWUPD_DEVICE_FLAG_UNKNOWN)
 		return "unknown";
 	return NULL;
@@ -138,10 +146,12 @@ fwupd_device_flag_from_string (const gchar *device_flag)
 		return FWUPD_DEVICE_FLAG_NONE;
 	if (g_strcmp0 (device_flag, "internal") == 0)
 		return FWUPD_DEVICE_FLAG_INTERNAL;
-	if (g_strcmp0 (device_flag, "allow-online") == 0)
-		return FWUPD_DEVICE_FLAG_ALLOW_ONLINE;
-	if (g_strcmp0 (device_flag, "allow-offline") == 0)
-		return FWUPD_DEVICE_FLAG_ALLOW_OFFLINE;
+	if (g_strcmp0 (device_flag, "updatable") == 0 ||
+	    g_strcmp0 (device_flag, "allow-online") == 0)
+		return FWUPD_DEVICE_FLAG_UPDATABLE;
+	if (g_strcmp0 (device_flag, "only-offline") == 0 ||
+	    g_strcmp0 (device_flag, "allow-offline") == 0)
+		return FWUPD_DEVICE_FLAG_ONLY_OFFLINE;
 	if (g_strcmp0 (device_flag, "require-ac") == 0)
 		return FWUPD_DEVICE_FLAG_REQUIRE_AC;
 	if (g_strcmp0 (device_flag, "locked") == 0)
@@ -150,6 +160,10 @@ fwupd_device_flag_from_string (const gchar *device_flag)
 		return FWUPD_DEVICE_FLAG_SUPPORTED;
 	if (g_strcmp0 (device_flag, "needs-bootloader") == 0)
 		return FWUPD_DEVICE_FLAG_NEEDS_BOOTLOADER;
+	if (g_strcmp0 (device_flag, "registered") == 0)
+		return FWUPD_DEVICE_FLAG_REGISTERED;
+	if (g_strcmp0 (device_flag, "needs-reboot") == 0)
+		return FWUPD_DEVICE_FLAG_NEEDS_REBOOT;
 	return FWUPD_DEVICE_FLAG_UNKNOWN;
 }
 
@@ -244,3 +258,47 @@ fwupd_trust_flag_from_string (const gchar *trust_flag)
 		return FWUPD_TRUST_FLAG_METADATA;
 	return FWUPD_TRUST_FLAG_LAST;
 }
+
+/**
+ * fwupd_keyring_kind_from_string:
+ * @keyring_kind: a string, e.g. "gpg"
+ *
+ * Converts an printable string to an enumerated type.
+ *
+ * Returns: a #FwupdKeyringKind, e.g. %FWUPD_KEYRING_KIND_GPG
+ *
+ * Since: 0.9.7
+ **/
+FwupdKeyringKind
+fwupd_keyring_kind_from_string (const gchar *keyring_kind)
+{
+	if (g_strcmp0 (keyring_kind, "none") == 0)
+		return FWUPD_KEYRING_KIND_NONE;
+	if (g_strcmp0 (keyring_kind, "gpg") == 0)
+		return FWUPD_KEYRING_KIND_GPG;
+	if (g_strcmp0 (keyring_kind, "pkcs7") == 0)
+		return FWUPD_KEYRING_KIND_PKCS7;
+	return FWUPD_KEYRING_KIND_UNKNOWN;
+}
+
+/**
+ * fwupd_keyring_kind_to_string:
+ * @keyring_kind: a #FwupdKeyringKind, e.g. %FWUPD_KEYRING_KIND_GPG
+ *
+ * Converts an enumerated type to a printable string.
+ *
+ * Returns: a string, e.g. "gpg"
+ *
+ * Since: 0.9.7
+ **/
+const gchar *
+fwupd_keyring_kind_to_string (FwupdKeyringKind keyring_kind)
+{
+	if (keyring_kind == FWUPD_KEYRING_KIND_NONE)
+		return "none";
+	if (keyring_kind == FWUPD_KEYRING_KIND_GPG)
+		return "gpg";
+	if (keyring_kind == FWUPD_KEYRING_KIND_PKCS7)
+		return "pkcs7";
+	return NULL;
+}
