@@ -24,12 +24,6 @@
 
 #include <glib.h>
 
-#define FWUPD_DBUS_PATH			"/"
-#define FWUPD_DBUS_SERVICE		"org.freedesktop.fwupd"
-#define FWUPD_DBUS_INTERFACE		"org.freedesktop.fwupd"
-
-#define FWUPD_DEVICE_ID_ANY		"*"
-
 /**
  * FwupdStatus:
  * @FWUPD_STATUS_UNKNOWN:			Unknown state
@@ -40,6 +34,7 @@
  * @FWUPD_STATUS_DEVICE_WRITE:			Writing to a device
  * @FWUPD_STATUS_DEVICE_VERIFY:			Verifying (reading) a device
  * @FWUPD_STATUS_SCHEDULING:			Scheduling an offline update
+ * @FWUPD_STATUS_DOWNLOADING:			A file is downloading
  *
  * The flags to show daemon status.
  **/
@@ -52,6 +47,7 @@ typedef enum {
 	FWUPD_STATUS_DEVICE_WRITE,			/* Since: 0.1.1 */
 	FWUPD_STATUS_DEVICE_VERIFY,			/* Since: 0.1.1 */
 	FWUPD_STATUS_SCHEDULING,			/* Since: 0.1.1 */
+	FWUPD_STATUS_DOWNLOADING,			/* Since: 0.9.4 */
 	/*< private >*/
 	FWUPD_STATUS_LAST
 } FwupdStatus;
@@ -76,12 +72,14 @@ typedef enum {
  * FwupdDeviceFlags:
  * @FWUPD_DEVICE_FLAG_NONE:			No flags set
  * @FWUPD_DEVICE_FLAG_INTERNAL:			Device cannot be removed easily
- * @FWUPD_DEVICE_FLAG_ALLOW_ONLINE:		Is capable of 'live' updating
- * @FWUPD_DEVICE_FLAG_ALLOW_OFFLINE:		Is capable of 'offline' updating
+ * @FWUPD_DEVICE_FLAG_UPDATABLE:		Device is updatable
+ * @FWUPD_DEVICE_FLAG_ONLY_OFFLINE:		Update can only be done from offline mode
  * @FWUPD_DEVICE_FLAG_REQUIRE_AC:		Requires AC power
  * @FWUPD_DEVICE_FLAG_LOCKED:			Is locked and can be unlocked
  * @FWUPD_DEVICE_FLAG_SUPPORTED:		Is found in current metadata
  * @FWUPD_DEVICE_FLAG_NEEDS_BOOTLOADER:		Requires a bootloader mode
+ * @FWUPD_DEVICE_FLAG_REGISTERED:		Has been registered with other plugins
+ * @FWUPD_DEVICE_FLAG_NEEDS_REBOOT:		Requires a reboot to apply firmware or to reload hardware
  *
  * FIXME: rename FU_DEVICE_ -> FWUPD_DEVICE_ when we break API
  *
@@ -89,30 +87,34 @@ typedef enum {
  **/
 #define FWUPD_DEVICE_FLAG_NONE			(0u)		/* Since: 0.1.3 */
 #define FWUPD_DEVICE_FLAG_INTERNAL		(1u << 0)	/* Since: 0.1.3 */
-#define FWUPD_DEVICE_FLAG_ALLOW_ONLINE		(1u << 1)	/* Since: 0.1.3 */
-#define FWUPD_DEVICE_FLAG_ALLOW_OFFLINE		(1u << 2)	/* Since: 0.1.3 */
+#define FWUPD_DEVICE_FLAG_UPDATABLE		(1u << 1)	/* Since: 0.9.7 */
+#define FWUPD_DEVICE_FLAG_ONLY_OFFLINE		(1u << 2)	/* Since: 0.9.7 */
 #define FWUPD_DEVICE_FLAG_REQUIRE_AC		(1u << 3)	/* Since: 0.6.3 */
 #define FWUPD_DEVICE_FLAG_LOCKED		(1u << 4)	/* Since: 0.6.3 */
 #define FWUPD_DEVICE_FLAG_SUPPORTED		(1u << 5)	/* Since: 0.7.1 */
 #define FWUPD_DEVICE_FLAG_NEEDS_BOOTLOADER	(1u << 6)	/* Since: 0.7.3 */
+#define FWUPD_DEVICE_FLAG_REGISTERED		(1u << 7)	/* Since: 0.9.7 */
+#define FWUPD_DEVICE_FLAG_NEEDS_REBOOT		(1u << 8)	/* Since: 0.9.7 */
 #define FWUPD_DEVICE_FLAG_UNKNOWN		G_MAXUINT64	/* Since: 0.7.3 */
 typedef guint64 FwupdDeviceFlags;
 
 /* deprecated names */
 #define FU_DEVICE_FLAG_NONE		FWUPD_DEVICE_FLAG_NONE
 #define FU_DEVICE_FLAG_INTERNAL		FWUPD_DEVICE_FLAG_INTERNAL
-#define FU_DEVICE_FLAG_ALLOW_ONLINE	FWUPD_DEVICE_FLAG_ALLOW_ONLINE
-#define FU_DEVICE_FLAG_ALLOW_OFFLINE	FWUPD_DEVICE_FLAG_ALLOW_OFFLINE
+#define FU_DEVICE_FLAG_ALLOW_ONLINE	FWUPD_DEVICE_FLAG_UPDATABLE
+#define FU_DEVICE_FLAG_ALLOW_OFFLINE	FWUPD_DEVICE_FLAG_ONLY_OFFLINE
 #define FU_DEVICE_FLAG_REQUIRE_AC	FWUPD_DEVICE_FLAG_REQUIRE_AC
 #define FU_DEVICE_FLAG_LOCKED		FWUPD_DEVICE_FLAG_LOCKED
 #define FU_DEVICE_FLAG_SUPPORTED	FWUPD_DEVICE_FLAG_SUPPORTED
 #define FU_DEVICE_FLAG_NEEDS_BOOTLOADER	FWUPD_DEVICE_FLAG_NEEDS_BOOTLOADER
 #define FU_DEVICE_FLAG_UNKNOWN		FWUPD_DEVICE_FLAG_UNKNOWN
+#define FWUPD_DEVICE_FLAG_ALLOW_ONLINE	FWUPD_DEVICE_FLAG_UPDATABLE
+#define FWUPD_DEVICE_FLAG_ALLOW_OFFLINE	FWUPD_DEVICE_FLAG_ONLY_OFFLINE
 
 /**
  * FwupdInstallFlags:
  * @FWUPD_INSTALL_FLAG_NONE:			No flags set
- * @FWUPD_INSTALL_FLAG_OFFLINE:			Perform this offline
+ * @FWUPD_INSTALL_FLAG_OFFLINE:			Schedule this for next boot
  * @FWUPD_INSTALL_FLAG_ALLOW_REINSTALL:		Allow reinstalling the same version
  * @FWUPD_INSTALL_FLAG_ALLOW_OLDER:		Allow downgrading firmware
  *
@@ -146,6 +148,24 @@ typedef enum {
 	FWUPD_UPDATE_STATE_LAST
 } FwupdUpdateState;
 
+/**
+ * FwupdKeyringKind:
+ * @FWUPD_KEYRING_KIND_UNKNOWN:			Unknown
+ * @FWUPD_KEYRING_KIND_NONE:			No verification
+ * @FWUPD_KEYRING_KIND_GPG:			Verification using GPG
+ * @FWUPD_KEYRING_KIND_PKCS7:			Verification using PKCS7
+ *
+ * The update state.
+ **/
+typedef enum {
+	FWUPD_KEYRING_KIND_UNKNOWN,			/* Since: 0.9.7 */
+	FWUPD_KEYRING_KIND_NONE,			/* Since: 0.9.7 */
+	FWUPD_KEYRING_KIND_GPG,				/* Since: 0.9.7 */
+	FWUPD_KEYRING_KIND_PKCS7,			/* Since: 0.9.7 */
+	/*< private >*/
+	FWUPD_KEYRING_KIND_LAST
+} FwupdKeyringKind;
+
 const gchar	*fwupd_status_to_string			(FwupdStatus	 status);
 FwupdStatus	 fwupd_status_from_string		(const gchar	*status);
 const gchar	*fwupd_device_flag_to_string		(FwupdDeviceFlags device_flag);
@@ -154,5 +174,7 @@ const gchar	*fwupd_update_state_to_string		(FwupdUpdateState update_state);
 FwupdUpdateState fwupd_update_state_from_string		(const gchar	*update_state);
 const gchar	*fwupd_trust_flag_to_string		(FwupdTrustFlags trust_flag);
 FwupdTrustFlags	 fwupd_trust_flag_from_string		(const gchar	*trust_flag);
+FwupdKeyringKind fwupd_keyring_kind_from_string		(const gchar	*keyring_kind);
+const gchar	*fwupd_keyring_kind_to_string		(FwupdKeyringKind keyring_kind);
 
 #endif /* __FWUPD_ENUMS_H */
