@@ -30,8 +30,12 @@
 #define __FWUPD_H_INSIDE__
 
 #include <libfwupd/fwupd-client.h>
+#include <libfwupd/fwupd-common.h>
+#include <libfwupd/fwupd-device.h>
 #include <libfwupd/fwupd-enums.h>
 #include <libfwupd/fwupd-error.h>
+#include <libfwupd/fwupd-release.h>
+#include <libfwupd/fwupd-remote.h>
 #include <libfwupd/fwupd-result.h>
 #include <libfwupd/fwupd-version.h>
 
