@@ -749,7 +749,7 @@ fu_device_altos_probe_bootloader (FuDeviceAltos *device, GError **error)
 		}
 
 		/* unknown line */
-		g_warning ("unknown data: '%s'", lines[i]);
+		g_debug ("unknown data: '%s'", lines[i]);
 	}
 
 	/* done */
@@ -811,14 +811,19 @@ fu_device_altos_init_real (FuDeviceAltos *device)
 {
 	FuDeviceAltosPrivate *priv = GET_PRIVATE (device);
 	g_autofree gchar *devid1 = NULL;
+	g_autofree gchar *vendor_id = NULL;
 
 	/* allowed, but requires manual bootloader step */
 	fu_device_add_flag (FU_DEVICE (device),
-			    FWUPD_DEVICE_FLAG_ALLOW_ONLINE);
+			    FWUPD_DEVICE_FLAG_UPDATABLE);
 
 	/* set default vendor */
 	fu_device_set_vendor (FU_DEVICE (device), "altusmetrum.org");
 
+	/* set vendor ID */
+	vendor_id = g_strdup_printf ("USB:0x%04X", g_usb_device_get_vid (priv->usb_device));
+	fu_device_set_vendor_id (FU_DEVICE (device), vendor_id);
+
 	/* set name */
 	switch (priv->kind) {
 	case FU_DEVICE_ALTOS_KIND_BOOTLOADER:
