@@ -36,6 +36,7 @@
 #include "fu-plugin-dell.h"
 #include "fu-quirks.h"
 #include "fu-plugin-vfuncs.h"
+#include "fu-device-metadata.h"
 
 /* These are used to indicate the status of a previous DELL flash */
 #define DELL_SUCCESS			0x0000
@@ -64,15 +65,6 @@ typedef struct _DOCK_DESCRIPTION
 	const gchar *		desc;
 } DOCK_DESCRIPTION;
 
-static void
-_dell_smi_obj_free (FuDellSmiObj *obj)
-{
-	dell_smi_obj_free (obj->smi);
-	g_free (obj);
-}
-
-G_DEFINE_AUTOPTR_CLEANUP_FUNC (FuDellSmiObj, _dell_smi_obj_free);
-
 /* These are for matching the components */
 #define WD15_EC_STR		"2 0 2 2 0"
 #define TB16_EC_STR		"2 0 2 1 0"
@@ -102,17 +94,18 @@ G_DEFINE_AUTOPTR_CLEANUP_FUNC (FuDellSmiObj, _dell_smi_obj_free);
 #define TBT_CBL_DESC		"Thunderbolt Cable"
 
 /**
- * Devices that should explicitly disable modeswitching
+ * Devices that should allow modeswitching
  */
-static guint16 tpm_switch_blacklist[] = {0x06D6, 0x06E6, 0x06E7, 0x06EB, 0x06EA,
-					 0x0702};
-
-typedef struct {
-	FuDevice		*device;
-	FuPlugin		*plugin;
-} FuPluginDockItem;
-
-
+static guint16 tpm_switch_whitelist[] = {0x06F2, 0x06F3, 0x06DD, 0x06DE, 0x06DF,
+					 0x06DB, 0x06DC, 0x06BB, 0x06C6, 0x06BA,
+					 0x06B9, 0x05CA, 0x06C7, 0x06B7, 0x06E0,
+					 0x06E5, 0x06D9, 0x06DA, 0x06E4, 0x0704,
+					 0x0720, 0x0730, 0x0758, 0x0759, 0x075B,
+					 0x07A0, 0x079F, 0x07A4, 0x07A5, 0x07A6,
+					 0x07A7, 0x07A8, 0x07A9, 0x07AA, 0x07AB,
+					 0x07B0, 0x07B1, 0x07B2, 0x07B4, 0x07B7,
+					 0x07B8, 0x07B9, 0x07BE, 0x07BF, 0x077A,
+					 0x07CF};
 static void
 _fwup_resource_iter_free (fwup_resource_iter *iter)
 {
@@ -151,7 +144,7 @@ fu_plugin_dell_match_dock_component (const gchar *query_str,
 void
 fu_plugin_dell_inject_fake_data (FuPlugin *plugin,
 				 guint32 *output, guint16 vid, guint16 pid,
-				 guint8 *buf)
+				 guint8 *buf, gboolean can_switch_modes)
 {
 	FuPluginData *data = fu_plugin_get_data (plugin);
 
@@ -162,13 +155,7 @@ fu_plugin_dell_inject_fake_data (FuPlugin *plugin,
 	data->fake_vid = vid;
 	data->fake_pid = pid;
 	data->smi_obj->fake_buffer = buf;
-}
-
-static void
-fu_plugin_device_free (FuPluginDockItem *item)
-{
-	g_object_unref (item->device);
-	g_object_unref (item->plugin);
+	data->can_switch_modes = TRUE;
 }
 
 static AsVersionParseFlag
@@ -208,8 +195,25 @@ static gboolean
 fu_plugin_dell_capsule_supported (FuPlugin *plugin)
 {
 	FuPluginData *data = fu_plugin_get_data (plugin);
+	gint uefi_supported;
 
-	return data->smi_obj->fake_smbios || data->capsule_supported;
+	if (data->smi_obj->fake_smbios)
+		return TRUE;
+
+	/* If ESRT is not turned on, fwupd will have already created an
+	 * unlock device (if compiled with support).
+	 *
+	 * Once unlocked, that will enable flashing capsules here too.
+	 *
+	 * that means we should only look for supported = 1
+	 */
+	uefi_supported = fwup_supported ();
+	if (uefi_supported != 1) {
+		g_debug ("UEFI capsule firmware updating not supported (%x)",
+			 (guint) uefi_supported);
+		return FALSE;
+	}
+	return TRUE;
 }
 
 static gboolean
@@ -217,13 +221,12 @@ fu_plugin_dock_node (FuPlugin *plugin, GUsbDevice *device,
 		     guint8 type, const efi_guid_t *guid_raw,
 		     const gchar *component_desc, const gchar *version)
 {
-	FuPluginData *data = fu_plugin_get_data (plugin);
-	FuPluginDockItem *item;
 	const gchar *dock_type;
 	g_autofree gchar *dock_id = NULL;
 	g_autofree gchar *guid_str = NULL;
 	g_autofree gchar *dock_key = NULL;
 	g_autofree gchar *dock_name = NULL;
+	g_autoptr(FuDevice) dev = NULL;
 
 	dock_type = fu_dell_get_dock_type (type);
 	if (dock_type == NULL) {
@@ -239,31 +242,29 @@ fu_plugin_dock_node (FuPlugin *plugin, GUsbDevice *device,
 
 	dock_key = fu_plugin_get_dock_key (plugin, device,
 						  guid_str);
-	item = g_hash_table_lookup (data->devices, dock_key);
-	if (item != NULL) {
-		g_debug ("Item %s is already registered.", dock_key);
+	if (fu_plugin_cache_lookup (plugin, dock_key) != NULL) {
+		g_debug ("%s is already registered.", dock_key);
 		return FALSE;
 	}
 
-	item = g_new0 (FuPluginDockItem, 1);
-	item->plugin = g_object_ref (plugin);
-	item->device = fu_device_new ();
+	dev = fu_device_new ();
 	dock_id = g_strdup_printf ("DELL-%s" G_GUINT64_FORMAT, guid_str);
 	dock_name = g_strdup_printf ("Dell %s %s", dock_type,
 				     component_desc);
-	fu_device_set_id (item->device, dock_id);
-	fu_device_set_name (item->device, dock_name);
-	fu_device_add_guid (item->device, guid_str);
-	fu_device_add_flag (item->device, FWUPD_DEVICE_FLAG_REQUIRE_AC);
+	fu_device_set_id (dev, dock_id);
+	fu_device_set_name (dev, dock_name);
+	fu_device_add_guid (dev, guid_str);
+	fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_REQUIRE_AC);
 	if (version != NULL) {
-		fu_device_set_version (item->device, version);
-		if (fu_plugin_dell_capsule_supported (plugin))
-			fu_device_add_flag (item->device,
-					    FWUPD_DEVICE_FLAG_ALLOW_OFFLINE);
+		fu_device_set_version (dev, version);
+		if (fu_plugin_dell_capsule_supported (plugin)) {
+			fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_UPDATABLE);
+			fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_NEEDS_REBOOT);
+		}
 	}
 
-	g_hash_table_insert (data->devices, g_strdup (dock_key), item);
-	fu_plugin_device_add (plugin, item->device);
+	fu_plugin_device_add (plugin, dev);
+	fu_plugin_cache_add (plugin, dock_key, dev);
 	return TRUE;
 }
 
@@ -284,7 +285,6 @@ fu_plugin_dell_device_added_cb (GUsbContext *ctx,
 	efi_guid_t guid_raw;
 	efi_guid_t tmpguid;
 	gboolean old_ec = FALSE;
-	g_autofree gchar *fw_str = NULL;
 
 	/* don't look up immediately if a dock is connected as that would
 	   mean a SMI on every USB device that showed up on the system */
@@ -324,6 +324,7 @@ fu_plugin_dell_device_added_cb (GUsbContext *ctx,
 	parse_flags = fu_plugin_dell_get_version_format ();
 
 	for (guint i = 0; i < dock_info->component_count; i++) {
+		g_autofree gchar *fw_str = NULL;
 		if (i >= MAX_COMPONENTS) {
 			g_debug ("Too many components.  Invalid: #%u", i);
 			break;
@@ -332,14 +333,9 @@ fu_plugin_dell_device_added_cb (GUsbContext *ctx,
 			 dock_info->components[i].description,
 			 dock_info->components[i].fw_version);
 		query_str = g_strrstr (dock_info->components[i].description,
-				       "Query ");
-		if (query_str == NULL) {
-			g_debug ("Invalid dock component request");
-			return;
-		}
-		if (!fu_plugin_dell_match_dock_component (query_str + 6,
-							  &guid_raw,
-							  &component_name)) {
+				       "Query ") + 6;
+		if (!fu_plugin_dell_match_dock_component (query_str, &guid_raw,
+							    &component_name)) {
 			g_debug ("Unable to match dock component %s",
 				query_str);
 			return;
@@ -373,6 +369,7 @@ fu_plugin_dell_device_added_cb (GUsbContext *ctx,
 
 	/* if an old EC or invalid EC version found, create updatable parent */
 	if (old_ec) {
+		g_autofree gchar *fw_str = NULL;
 		tmpguid = DOCK_FLASH_GUID;
 		fw_str = as_utils_version_from_uint32 (dock_info->flash_pkg_version,
 						       parse_flags);
@@ -394,8 +391,6 @@ fu_plugin_dell_device_removed_cb (GUsbContext *ctx,
 				  FuPlugin *plugin)
 {
 	FuPluginData *data = fu_plugin_get_data (plugin);
-	FuPluginDockItem *item;
-	g_autofree gchar *dock_key = NULL;
 	const efi_guid_t guids[] = { WD15_EC_GUID, TB16_EC_GUID, TB16_PC2_GUID,
 				     TB16_PC1_GUID, WD15_PC1_GUID,
 				     LEGACY_CBL_GUID, UNIV_CBL_GUID,
@@ -403,8 +398,7 @@ fu_plugin_dell_device_removed_cb (GUsbContext *ctx,
 	const efi_guid_t *guid_raw;
 	guint16 pid;
 	guint16 vid;
-
-	g_autofree gchar *guid_str = NULL;
+	FuDevice *dev = NULL;
 
 	if (!data->smi_obj->fake_smbios) {
 		vid = g_usb_device_get_vid (device);
@@ -420,16 +414,18 @@ fu_plugin_dell_device_removed_cb (GUsbContext *ctx,
 
 	/* remove any components already in database? */
 	for (guint i = 0; i < G_N_ELEMENTS (guids); i++) {
+		g_autofree gchar *dock_key = NULL;
+		g_autofree gchar *guid_str = NULL;
 		guid_raw = &guids[i];
 		guid_str = g_strdup ("00000000-0000-0000-0000-000000000000");
 		efi_guid_to_str (guid_raw, &guid_str);
 		dock_key = fu_plugin_get_dock_key (plugin, device,
 							  guid_str);
-		item = g_hash_table_lookup (data->devices, dock_key);
-		if (item != NULL) {
+		dev = fu_plugin_cache_lookup (plugin, dock_key);
+		if (dev != NULL) {
 			fu_plugin_device_remove (plugin,
-						   item->device);
-			g_hash_table_remove (data->devices, dock_key);
+						   dev);
+			fu_plugin_cache_remove (plugin, dock_key);
 		}
 	}
 }
@@ -509,7 +505,7 @@ fu_plugin_dell_detect_tpm (FuPlugin *plugin, GError **error)
 	const gchar *tpm_mode;
 	const gchar *tpm_mode_alt;
 	guint16 system_id = 0;
-	gboolean can_switch_modes = TRUE;
+	gboolean can_switch_modes = FALSE;
 	g_autofree gchar *pretty_tpm_name_alt = NULL;
 	g_autofree gchar *pretty_tpm_name = NULL;
 	g_autofree gchar *product_name = NULL;
@@ -565,10 +561,12 @@ fu_plugin_dell_detect_tpm (FuPlugin *plugin, GError **error)
 
 	if (!data->smi_obj->fake_smbios)
 		system_id = (guint16) sysinfo_get_dell_system_id ();
+	else
+		can_switch_modes = data->can_switch_modes;
 
-	for (guint i = 0; i < G_N_ELEMENTS (tpm_switch_blacklist); i++) {
-		if (tpm_switch_blacklist[i] == system_id) {
-			can_switch_modes = FALSE;
+	for (guint i = 0; i < G_N_ELEMENTS (tpm_switch_whitelist); i++) {
+		if (tpm_switch_whitelist[i] == system_id) {
+			can_switch_modes = TRUE;
 		}
 	}
 
@@ -608,13 +606,12 @@ fu_plugin_dell_detect_tpm (FuPlugin *plugin, GError **error)
 	fu_device_set_version (dev, version_str);
 	fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_INTERNAL);
 	fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_REQUIRE_AC);
-	if ((out->status & TPM_OWN_MASK) == 0 && out->flashes_left > 0) {
-		if (fu_plugin_dell_capsule_supported (plugin))
-			fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_ALLOW_OFFLINE);
+	if (out->flashes_left > 0) {
+		if (fu_plugin_dell_capsule_supported (plugin)) {
+			fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_UPDATABLE);
+			fu_device_add_flag (dev, FWUPD_DEVICE_FLAG_NEEDS_REBOOT);
+		}
 		fu_device_set_flashes_left (dev, out->flashes_left);
-	} else {
-		g_debug ("%s updating disabled due to TPM ownership",
-			pretty_tpm_name);
 	}
 	fu_plugin_device_add (plugin, dev);
 
@@ -634,7 +631,8 @@ fu_plugin_dell_detect_tpm (FuPlugin *plugin, GError **error)
 		 * Mode switching is turned on by setting flashes left on alternate
 		 * device.
 		 */
-		if ((out->status & TPM_OWN_MASK) == 0 && out->flashes_left > 0) {
+		if (!((out->status) & TPM_OWN_MASK) &&
+		    out->flashes_left > 0) {
 			fu_device_set_flashes_left (dev_alt, out->flashes_left);
 		} else {
 			g_debug ("%s mode switch disabled due to TPM ownership",
@@ -643,7 +641,7 @@ fu_plugin_dell_detect_tpm (FuPlugin *plugin, GError **error)
 		fu_plugin_device_add (plugin, dev_alt);
 	}
 	else
-		g_debug ("System %04x is on blacklist, disabling TPM modeswitch",
+		g_debug ("System %04x does not offer TPM modeswitching",
 			system_id);
 
 	return TRUE;
@@ -693,7 +691,7 @@ fu_plugin_unlock (FuPlugin *plugin, FuDevice *device, GError **error)
 	/* clone the info from real device but prevent it from being flashed */
 	device_flags_alt = fu_device_get_flags (device_alt);
 	fu_device_set_flags (device, device_flags_alt);
-	fu_device_set_flags (device_alt, device_flags_alt & ~FWUPD_DEVICE_FLAG_ALLOW_OFFLINE);
+	fu_device_set_flags (device_alt, device_flags_alt & ~FWUPD_DEVICE_FLAG_UPDATABLE);
 
 	/* make sure that this unlocked device can be updated */
 	fu_device_set_version (device, "0.0.0.0");
@@ -702,11 +700,11 @@ fu_plugin_unlock (FuPlugin *plugin, FuDevice *device, GError **error)
 }
 
 gboolean
-fu_plugin_update_offline (FuPlugin *plugin,
-			  FuDevice *device,
-			  GBytes *blob_fw,
-			  FwupdInstallFlags flags,
-			  GError **error)
+fu_plugin_update (FuPlugin *plugin,
+		  FuDevice *device,
+		  GBytes *blob_fw,
+		  FwupdInstallFlags flags,
+		  GError **error)
 {
 	FuPluginData *data = fu_plugin_get_data (plugin);
 	g_autoptr (fwup_resource_iter) iter = NULL;
@@ -733,7 +731,7 @@ fu_plugin_update_offline (FuPlugin *plugin,
 				     FWUPD_ERROR,
 				     FWUPD_ERROR_NOT_SUPPORTED,
 				     "WARNING: %s only has %u flashes left. "
-				     "To update anyway please run the update with --force.",
+				     "See https://github.com/hughsie/fwupd/wiki/Dell-TPM:-flashes-left for more information.",
 				     name, flashes_left);
 			return FALSE;
 		}
@@ -784,25 +782,64 @@ fu_plugin_update_offline (FuPlugin *plugin,
 					  g_bytes_get_data (blob_fw, NULL),
 					  g_bytes_get_size (blob_fw));
 	if (rc < 0) {
-		g_set_error (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR_NOT_SUPPORTED,
-			     "DELL capsule update failed: %s",
-			     strerror (rc));
-		return FALSE;
-	}
+                g_autoptr(GString) err_string = g_string_new ("Dell firmware update failed:\n");
+
+                rc = 1;
+                for (int i =0; rc > 0; i++) {
+                        char *filename = NULL;
+                        char *function = NULL;
+                        char *message = NULL;
+                        int line = 0;
+                        int err = 0;
+
+                        rc = efi_error_get (i, &filename, &function, &line, &message, &err);
+                        if (rc <= 0)
+                                break;
+                        g_string_append_printf (err_string,
+                                                "{error #%d} %s:%d %s(): %s: %s \n",
+                                                i, filename, line, function, message, strerror(err));
+                }
+                g_set_error (error,
+                             FWUPD_ERROR,
+                             FWUPD_ERROR_NOT_SUPPORTED,
+                             "%s",
+                             err_string->str);
+                return FALSE;
+        }
 	return TRUE;
 }
 
+void
+fu_plugin_device_registered (FuPlugin *plugin, FuDevice *device)
+{
+	/* thunderbolt plugin */
+	if (g_strcmp0 (fu_device_get_plugin (device), "thunderbolt") == 0 &&
+	    fu_device_has_flag (device, FWUPD_DEVICE_FLAG_INTERNAL)) {
+		/* prevent thunderbolt controllers in the system from going away */
+		fu_device_set_metadata_boolean (device,
+						FU_DEVICE_METADATA_TBT_CAN_FORCE_POWER,
+						TRUE);
+		/* fix VID/DID of safe mode devices */
+		if (fu_device_get_metadata_boolean (device, FU_DEVICE_METADATA_TBT_IS_SAFE_MODE)) {
+			g_autofree gchar *vendor_id = NULL;
+			g_autofree gchar *device_id = NULL;
+			vendor_id = g_strdup ("TBT:0x00D4");
+			/* the kernel returns lowercase in sysfs, need to match it */
+			device_id = g_strdup_printf ("TBT-%04x%04x", 0x00d4u,
+						     (unsigned) sysinfo_get_dell_system_id ());
+			fu_device_set_vendor_id (device, vendor_id);
+			fu_device_add_guid (device, device_id);
+			fu_device_add_flag (device, FWUPD_DEVICE_FLAG_UPDATABLE);
+		}
+	}
+}
+
 gboolean
 fu_plugin_update_prepare (FuPlugin *plugin,
                           FuDevice *device,
                           GError **error)
 {
-	if (!fu_dell_toggle_flash (device, error, TRUE))
-		return FALSE;
-	g_usleep (DELL_FLASH_MODE_DELAY * G_USEC_PER_SEC);
-	return TRUE;
+	return fu_dell_toggle_flash (device, error, TRUE);
 }
 
 gboolean
@@ -829,8 +866,6 @@ void
 fu_plugin_init (FuPlugin *plugin)
 {
 	FuPluginData *data = fu_plugin_alloc_data (plugin, sizeof (FuPluginData));
-	data->devices = g_hash_table_new_full (g_str_hash, g_str_equal,
-					       g_free, (GDestroyNotify) fu_plugin_device_free);
 
 	data->smi_obj = g_malloc0 (sizeof (FuDellSmiObj));
 	if (fu_dell_supported ())
@@ -844,7 +879,6 @@ void
 fu_plugin_destroy (FuPlugin *plugin)
 {
 	FuPluginData *data = fu_plugin_get_data (plugin);
-	g_hash_table_unref (data->devices);
 	if (data->smi_obj->smi)
 		dell_smi_obj_free (data->smi_obj->smi);
 	g_free(data->smi_obj);
@@ -855,7 +889,6 @@ fu_plugin_startup (FuPlugin *plugin, GError **error)
 {
 	FuPluginData *data = fu_plugin_get_data (plugin);
 	GUsbContext *usb_ctx = fu_plugin_get_usb_context (plugin);
-	gint uefi_supported;
 
 	if (data->smi_obj->fake_smbios) {
 		g_debug ("Called with fake SMBIOS implementation. "
@@ -872,28 +905,6 @@ fu_plugin_startup (FuPlugin *plugin, GError **error)
 		return FALSE;
 	}
 
-	if (data->smi_obj->smi == NULL) {
-		g_set_error (error,
-			     FWUPD_ERROR,
-			     FWUPD_ERROR_INTERNAL,
-			     "failed to initialize libsmbios library");
-		return FALSE;
-	}
-
-	/* If ESRT is not turned on, fwupd will have already created an
-	 * unlock device (if compiled with support).
-	 *
-	 * Once unlocked, that will enable flashing capsules here too.
-	 *
-	 * that means we should only look for supported = 1
-	 */
-	uefi_supported = fwup_supported ();
-	data->capsule_supported = (uefi_supported == 1);
-	if (!data->capsule_supported) {
-		g_debug ("UEFI capsule firmware updating not supported (%x)",
-			 (guint) uefi_supported);
-	}
-
 	if (usb_ctx != NULL) {
 		g_signal_connect (usb_ctx, "device-added",
 				  G_CALLBACK (fu_plugin_dell_device_added_cb),
