@@ -27,17 +27,16 @@
 #include "fu-dell-common.h"
 
 struct FuPluginData {
-	GHashTable		*devices;	/* DeviceKey:FuPluginDockItem */
 	FuDellSmiObj		*smi_obj;
 	guint16			fake_vid;
 	guint16			fake_pid;
-	gboolean		capsule_supported;
+	gboolean		can_switch_modes;
 };
 
 void
 fu_plugin_dell_inject_fake_data (FuPlugin *plugin,
 				 guint32 *output, guint16 vid, guint16 pid,
-				 guint8 *buf);
+				 guint8 *buf, gboolean can_switch_modes);
 
 gboolean
 fu_plugin_dell_detect_tpm (FuPlugin *plugin, GError **error);
