@@ -56,8 +56,7 @@ typedef struct {
 } FuRomPciHeader;
 
 typedef struct {
-	GChecksum			*checksum_wip;
-	GChecksumType			 checksum_type;
+	GPtrArray			*checksums;
 	GInputStream			*stream;
 	FuRomKind			 kind;
 	gchar				*version;
@@ -572,9 +571,9 @@ fu_rom_load_data (FuRom *rom,
 	gssize sz = buffer_sz;
 	guint32 jump = 0;
 	guint32 hdr_sz = 0;
-	g_autoptr(GError) error_local = NULL;
 	g_autofree gchar *id = NULL;
-	g_autoptr(AsProfile) profile = as_profile_new ();
+	g_autoptr(GChecksum) checksum_sha1 = g_checksum_new (G_CHECKSUM_SHA1);
+	g_autoptr(GChecksum) checksum_sha256 = g_checksum_new (G_CHECKSUM_SHA256);
 
 	g_return_val_if_fail (FU_IS_ROM (rom), FALSE);
 
@@ -701,9 +700,14 @@ fu_rom_load_data (FuRom *rom,
 		fu_rom_find_and_blank_serial_numbers (rom);
 	for (guint i = 0; i < priv->hdrs->len; i++) {
 		hdr = g_ptr_array_index (priv->hdrs, i);
-		g_checksum_update (priv->checksum_wip, hdr->rom_data, hdr->rom_len);
+		g_checksum_update (checksum_sha1, hdr->rom_data, hdr->rom_len);
+		g_checksum_update (checksum_sha256, hdr->rom_data, hdr->rom_len);
 	}
 
+	/* done updating checksums */
+	g_ptr_array_add (priv->checksums, g_strdup (g_checksum_get_string (checksum_sha1)));
+	g_ptr_array_add (priv->checksums, g_strdup (g_checksum_get_string (checksum_sha256)));
+
 	/* update guid */
 	id = g_strdup_printf ("PCI\\VEN_%04X&DEV_%04X",
 			      priv->vendor_id, priv->device_id);
@@ -847,18 +851,11 @@ fu_rom_get_model (FuRom *rom)
 	return priv->device_id;
 }
 
-const gchar *
-fu_rom_get_checksum (FuRom *rom)
+GPtrArray *
+fu_rom_get_checksums (FuRom *rom)
 {
 	FuRomPrivate *priv = GET_PRIVATE (rom);
-	return g_checksum_get_string (priv->checksum_wip);
-}
-
-GChecksumType
-fu_rom_get_checksum_kind (FuRom *rom)
-{
-	FuRomPrivate *priv = GET_PRIVATE (rom);
-	return priv->checksum_type;
+	return priv->checksums;
 }
 
 static void
@@ -872,8 +869,7 @@ static void
 fu_rom_init (FuRom *rom)
 {
 	FuRomPrivate *priv = GET_PRIVATE (rom);
-	priv->checksum_type = G_CHECKSUM_SHA1;
-	priv->checksum_wip = g_checksum_new (priv->checksum_type);
+	priv->checksums = g_ptr_array_new_with_free_func (g_free);
 	priv->hdrs = g_ptr_array_new_with_free_func ((GDestroyNotify) fu_rom_pci_header_free);
 }
 
@@ -883,9 +879,9 @@ fu_rom_finalize (GObject *object)
 	FuRom *rom = FU_ROM (object);
 	FuRomPrivate *priv = GET_PRIVATE (rom);
 
-	g_checksum_free (priv->checksum_wip);
 	g_free (priv->version);
 	g_free (priv->guid);
+	g_ptr_array_unref (priv->checksums);
 	g_ptr_array_unref (priv->hdrs);
 	if (priv->stream != NULL)
 		g_object_unref (priv->stream);
