@@ -29,11 +29,6 @@
 
 static void fu_device_finalize			 (GObject *object);
 
-/**
- * FuDevicePrivate:
- *
- * Private #FuDevice data
- **/
 typedef struct {
 	gchar				*equivalent_id;
 	FuDevice			*alternate;
@@ -43,9 +38,6 @@ typedef struct {
 G_DEFINE_TYPE_WITH_PRIVATE (FuDevice, fu_device, FWUPD_TYPE_RESULT)
 #define GET_PRIVATE(o) (fu_device_get_instance_private (o))
 
-/**
- * fu_device_get_equivalent_id:
- **/
 const gchar *
 fu_device_get_equivalent_id (FuDevice *device)
 {
@@ -54,9 +46,6 @@ fu_device_get_equivalent_id (FuDevice *device)
 	return priv->equivalent_id;
 }
 
-/**
- * fu_device_set_equivalent_id:
- **/
 void
 fu_device_set_equivalent_id (FuDevice *device, const gchar *equivalent_id)
 {
@@ -66,9 +55,6 @@ fu_device_set_equivalent_id (FuDevice *device, const gchar *equivalent_id)
 	priv->equivalent_id = g_strdup (equivalent_id);
 }
 
-/**
- * fu_device_get_alternate:
- **/
 FuDevice *
 fu_device_get_alternate (FuDevice *device)
 {
@@ -77,9 +63,6 @@ fu_device_get_alternate (FuDevice *device)
 	return priv->alternate;
 }
 
-/**
- * fu_device_set_alternate:
- **/
 void
 fu_device_set_alternate (FuDevice *device, FuDevice *alternate)
 {
@@ -88,9 +71,6 @@ fu_device_set_alternate (FuDevice *device, FuDevice *alternate)
 	g_set_object (&priv->alternate, alternate);
 }
 
-/**
- * fu_device_add_guid:
- **/
 void
 fu_device_add_guid (FuDevice *device, const gchar *guid)
 {
@@ -98,16 +78,24 @@ fu_device_add_guid (FuDevice *device, const gchar *guid)
 	if (!as_utils_guid_is_valid (guid)) {
 		g_autofree gchar *tmp = as_utils_guid_from_string (guid);
 		g_debug ("using %s for %s", tmp, guid);
-		fwupd_result_add_guid (FWUPD_RESULT (device), tmp);
+		fwupd_device_add_guid (fwupd_result_get_device (FWUPD_RESULT (device)), tmp);
 		return;
 	}
 
 	/* already valid */
-	fwupd_result_add_guid (FWUPD_RESULT (device), guid);
+	fwupd_device_add_guid (fwupd_result_get_device (FWUPD_RESULT (device)), guid);
 }
 
 /**
  * fu_device_get_metadata:
+ * @device: A #FuDevice
+ * @key: the key
+ *
+ * Gets an item of metadata from the device.
+ *
+ * Returns: a string value, or %NULL for unfound.
+ *
+ * Since: 0.1.0
  **/
 const gchar *
 fu_device_get_metadata (FuDevice *device, const gchar *key)
@@ -118,8 +106,72 @@ fu_device_get_metadata (FuDevice *device, const gchar *key)
 	return g_hash_table_lookup (priv->metadata, key);
 }
 
+/**
+ * fu_device_get_metadata_boolean:
+ * @device: A #FuDevice
+ * @key: the key
+ *
+ * Gets an item of metadata from the device.
+ *
+ * Returns: a boolean value, or %FALSE for unfound or failure to parse.
+ *
+ * Since: 0.9.7
+ **/
+gboolean
+fu_device_get_metadata_boolean (FuDevice *device, const gchar *key)
+{
+	FuDevicePrivate *priv = GET_PRIVATE (device);
+	const gchar *tmp;
+	g_return_val_if_fail (FU_IS_DEVICE (device), FALSE);
+	g_return_val_if_fail (key != NULL, FALSE);
+	tmp = g_hash_table_lookup (priv->metadata, key);
+	if (tmp == NULL)
+		return FALSE;
+	return g_strcmp0 (tmp, "true") == 0;
+}
+
+/**
+ * fu_device_get_metadata_integer:
+ * @device: A #FuDevice
+ * @key: the key
+ *
+ * Gets an item of metadata from the device.
+ *
+ * Returns: a string value, or %G_MAXUINT for unfound or failure to parse.
+ *
+ * Since: 0.9.7
+ **/
+guint
+fu_device_get_metadata_integer (FuDevice *device, const gchar *key)
+{
+	FuDevicePrivate *priv = GET_PRIVATE (device);
+	const gchar *tmp;
+	gchar *endptr = NULL;
+	guint64 val;
+
+	g_return_val_if_fail (FU_IS_DEVICE (device), G_MAXUINT);
+	g_return_val_if_fail (key != NULL, G_MAXUINT);
+
+	tmp = g_hash_table_lookup (priv->metadata, key);
+	if (tmp == NULL)
+		return G_MAXUINT;
+	val = g_ascii_strtoull (tmp, &endptr, 10);
+	if (endptr != NULL && endptr[0] != '\0')
+		return G_MAXUINT;
+	if (val > G_MAXUINT)
+		return G_MAXUINT;
+	return (guint) val;
+}
+
 /**
  * fu_device_set_metadata:
+ * @device: A #FuDevice
+ * @key: the key
+ * @value: the string value
+ *
+ * Sets an item of metadata on the device.
+ *
+ * Since: 0.1.0
  **/
 void
 fu_device_set_metadata (FuDevice *device, const gchar *key, const gchar *value)
@@ -132,20 +184,49 @@ fu_device_set_metadata (FuDevice *device, const gchar *key, const gchar *value)
 }
 
 /**
- * fu_device_set_name:
+ * fu_device_set_metadata_boolean:
+ * @device: A #FuDevice
+ * @key: the key
+ * @value: the boolean value
+ *
+ * Sets an item of metadata on the device. When @value is set to %TRUE
+ * the actual stored value is "true".
+ *
+ * Since: 0.9.7
  **/
+void
+fu_device_set_metadata_boolean (FuDevice *device, const gchar *key, gboolean value)
+{
+	fu_device_set_metadata (device, key, value ? "true" : "false");
+}
+
+/**
+ * fu_device_set_metadata_integer:
+ * @device: A #FuDevice
+ * @key: the key
+ * @value: the unsigned integer value
+ *
+ * Sets an item of metadata on the device. The integer is stored as a
+ * base-10 string internally.
+ *
+ * Since: 0.9.7
+ **/
+void
+fu_device_set_metadata_integer (FuDevice *device, const gchar *key, guint value)
+{
+	g_autofree gchar *tmp = g_strdup_printf ("%u", value);
+	fu_device_set_metadata (device, key, tmp);
+}
+
 void
 fu_device_set_name (FuDevice *device, const gchar *value)
 {
 	g_autoptr(GString) new = g_string_new (value);
 	g_strdelimit (new->str, "_", ' ');
 	as_utils_string_replace (new, "(TM)", "™");
-	fwupd_result_set_device_name (FWUPD_RESULT (device), new->str);
+	fwupd_device_set_name (fwupd_result_get_device (FWUPD_RESULT (device)), new->str);
 }
 
-/**
- * fu_device_class_init:
- **/
 static void
 fu_device_class_init (FuDeviceClass *klass)
 {
@@ -153,9 +234,6 @@ fu_device_class_init (FuDeviceClass *klass)
 	object_class->finalize = fu_device_finalize;
 }
 
-/**
- * fu_device_init:
- **/
 static void
 fu_device_init (FuDevice *device)
 {
@@ -164,9 +242,6 @@ fu_device_init (FuDevice *device)
 						g_free, g_free);
 }
 
-/**
- * fu_device_finalize:
- **/
 static void
 fu_device_finalize (GObject *object)
 {
@@ -180,9 +255,6 @@ fu_device_finalize (GObject *object)
 	G_OBJECT_CLASS (fu_device_parent_class)->finalize (object);
 }
 
-/**
- * fu_device_new:
- **/
 FuDevice *
 fu_device_new (void)
 {
