@@ -31,11 +31,6 @@
 
 static void fu_pending_finalize			 (GObject *object);
 
-/**
- * FuPendingPrivate:
- *
- * Private #FuPending data
- **/
 typedef struct {
 	sqlite3				*db;
 } FuPendingPrivate;
@@ -43,9 +38,6 @@ typedef struct {
 G_DEFINE_TYPE_WITH_PRIVATE (FuPending, fu_pending, G_TYPE_OBJECT)
 #define GET_PRIVATE(o) (fu_pending_get_instance_private (o))
 
-/**
- * fu_pending_load:
- **/
 static gboolean
 fu_pending_load (FuPending *pending, GError **error)
 {
@@ -90,6 +82,7 @@ fu_pending_load (FuPending *pending, GError **error)
 		sqlite3_free (error_msg);
 		statement = "CREATE TABLE pending ("
 			    "device_id TEXT PRIMARY KEY,"
+			    "unique_id TEXT,"
 			    "state INTEGER DEFAULT 0,"
 			    "timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,"
 			    "error TEXT,"
@@ -133,20 +126,30 @@ fu_pending_load (FuPending *pending, GError **error)
 		g_debug ("FuPending: altering table to repair: %s", error_msg);
 		sqlite3_free (error_msg);
 		statement = "ALTER TABLE pending ADD COLUMN timestamp TIMESTAMP "
-			    "DEFAULT CURRENT_TIMESTAMP NOT NULL0;";
+			    "DEFAULT CURRENT_TIMESTAMP NOT NULL;";
+		sqlite3_exec (priv->db, statement, NULL, NULL, NULL);
+	}
+
+	/* check pending has unique_id (since 0.7.3) */
+	rc = sqlite3_exec (priv->db,
+			   "SELECT unique_id FROM pending LIMIT 1",
+			   NULL, NULL, &error_msg);
+	if (rc != SQLITE_OK) {
+		g_debug ("FuPending: altering table to repair: %s", error_msg);
+		sqlite3_free (error_msg);
+		statement = "ALTER TABLE pending ADD COLUMN unique_id TEXT;";
 		sqlite3_exec (priv->db, statement, NULL, NULL, NULL);
 	}
 
 	return TRUE;
 }
 
-/**
- * fu_pending_add_device:
- **/
 gboolean
 fu_pending_add_device (FuPending *pending, FwupdResult *res, GError **error)
 {
 	FuPendingPrivate *priv = GET_PRIVATE (pending);
+	FwupdDevice *dev = fwupd_result_get_device (res);
+	FwupdRelease *rel = fwupd_result_get_release (res);
 	char *error_msg = NULL;
 	char *statement;
 	gboolean ret = TRUE;
@@ -160,22 +163,24 @@ fu_pending_add_device (FuPending *pending, FwupdResult *res, GError **error)
 			return FALSE;
 	}
 
-	g_debug ("FuPending: add device %s", fwupd_result_get_device_id (res));
+	g_debug ("FuPending: add device %s", fwupd_device_get_id (dev));
 	statement = sqlite3_mprintf ("INSERT INTO pending (device_id,"
+							  "unique_id,"
 							  "state,"
 							  "filename,"
 							  "display_name,"
 							  "provider,"
 							  "version_old,"
 							  "version_new) "
-				     "VALUES ('%q','%i','%q','%q','%q','%q','%q')",
-				     fwupd_result_get_device_id (res),
+				     "VALUES ('%q','%q','%i','%q','%q','%q','%q','%q')",
+				     fwupd_device_get_id (dev),
+				     fwupd_result_get_unique_id (res),
 				     FWUPD_UPDATE_STATE_PENDING,
-				     fwupd_result_get_update_filename (res),
-				     fwupd_result_get_device_name (res),
-				     fwupd_result_get_device_provider (res),
-				     fwupd_result_get_device_version (res),
-				     fwupd_result_get_update_version (res));
+				     fwupd_release_get_filename (rel),
+				     fwupd_device_get_name (dev),
+				     fwupd_device_get_provider (dev),
+				     fwupd_device_get_version (dev),
+				     fwupd_release_get_version (rel));
 
 	/* insert entry */
 	rc = sqlite3_exec (priv->db, statement, NULL, NULL, &error_msg);
@@ -194,13 +199,11 @@ out:
 	return ret;
 }
 
-/**
- * fu_pending_remove_device:
- **/
 gboolean
 fu_pending_remove_device (FuPending *pending, FwupdResult *res, GError **error)
 {
 	FuPendingPrivate *priv = GET_PRIVATE (pending);
+	FwupdDevice *dev = fwupd_result_get_device (res);
 	char *error_msg = NULL;
 	char *statement;
 	gboolean ret = TRUE;
@@ -214,10 +217,10 @@ fu_pending_remove_device (FuPending *pending, FwupdResult *res, GError **error)
 			return FALSE;
 	}
 
-	g_debug ("FuPending: remove device %s", fwupd_result_get_device_id (res));
+	g_debug ("FuPending: remove device %s", fwupd_device_get_id (dev));
 	statement = sqlite3_mprintf ("DELETE FROM pending WHERE "
 				     "device_id = '%q';",
-				     fwupd_result_get_device_id (res));
+				     fwupd_device_get_id (dev));
 
 	/* remove entry */
 	rc = sqlite3_exec (priv->db, statement, NULL, NULL, &error_msg);
@@ -236,9 +239,6 @@ out:
 	return ret;
 }
 
-/**
- * fu_pending_device_sqlite_cb:
- **/
 static gint
 fu_pending_device_sqlite_cb (void *data,
 			gint argc,
@@ -246,37 +246,44 @@ fu_pending_device_sqlite_cb (void *data,
 			gchar **col_name)
 {
 	GPtrArray *array = (GPtrArray *) data;
+	FwupdDevice *dev;
+	FwupdRelease *rel;
 	FwupdResult *res;
-	gint i;
 
 	/* create new result */
 	res = fwupd_result_new ();
+	rel = fwupd_result_get_release (res);
+	dev = fwupd_result_get_device (res);
 	g_ptr_array_add (array, res);
 
 	g_debug ("FuPending: got sql result %s", argv[0]);
-	for (i = 0; i < argc; i++) {
+	for (gint i = 0; i < argc; i++) {
 		if (g_strcmp0 (col_name[i], "device_id") == 0) {
-			fwupd_result_set_device_id (res, argv[i]);
+			fwupd_device_set_id (dev, argv[i]);
+			continue;
+		}
+		if (g_strcmp0 (col_name[i], "unique_id") == 0) {
+			fwupd_result_set_unique_id (res, argv[i]);
 			continue;
 		}
 		if (g_strcmp0 (col_name[i], "filename") == 0) {
-			fwupd_result_set_update_filename (res, argv[i]);
+			fwupd_release_set_filename (rel, argv[i]);
 			continue;
 		}
 		if (g_strcmp0 (col_name[i], "display_name") == 0) {
-			fwupd_result_set_device_name (res, argv[i]);
+			fwupd_device_set_name (dev, argv[i]);
 			continue;
 		}
 		if (g_strcmp0 (col_name[i], "version_old") == 0) {
-			fwupd_result_set_device_version (res, argv[i]);
+			fwupd_device_set_version (dev, argv[i]);
 			continue;
 		}
 		if (g_strcmp0 (col_name[i], "version_new") == 0) {
-			fwupd_result_set_update_version (res, argv[i]);
+			fwupd_release_set_version (rel, argv[i]);
 			continue;
 		}
 		if (g_strcmp0 (col_name[i], "provider") == 0) {
-			fwupd_result_set_device_provider (res, argv[i]);
+			fwupd_device_set_provider (dev, argv[i]);
 			continue;
 		}
 		if (g_strcmp0 (col_name[i], "state") == 0) {
@@ -287,7 +294,7 @@ fu_pending_device_sqlite_cb (void *data,
 		if (g_strcmp0 (col_name[i], "timestamp") == 0) {
 			guint64 timestamp = g_ascii_strtoull (argv[i], NULL, 10);
 			if (timestamp > 0)
-				fwupd_result_set_device_created (res, timestamp);
+				fwupd_device_set_created (dev, timestamp);
 			continue;
 		}
 		if (g_strcmp0 (col_name[i], "error") == 0) {
@@ -295,15 +302,11 @@ fu_pending_device_sqlite_cb (void *data,
 				fwupd_result_set_update_error (res, argv[i]);
 			continue;
 		}
-		g_warning ("unhandled %s=%s", col_name[i], argv[i]);
 	}
 
 	return 0;
 }
 
-/**
- * fu_pending_get_device:
- **/
 FwupdResult *
 fu_pending_get_device (FuPending *pending, const gchar *device_id, GError **error)
 {
@@ -355,9 +358,6 @@ out:
 	return res;
 }
 
-/**
- * fu_pending_get_devices:
- **/
 GPtrArray *
 fu_pending_get_devices (FuPending *pending, GError **error)
 {
@@ -402,9 +402,6 @@ out:
 	return array;
 }
 
-/**
- * fu_pending_set_state:
- **/
 gboolean
 fu_pending_set_state (FuPending *pending,
 		      FwupdResult *res,
@@ -412,6 +409,7 @@ fu_pending_set_state (FuPending *pending,
 		      GError **error)
 {
 	FuPendingPrivate *priv = GET_PRIVATE (pending);
+	FwupdDevice *dev = fwupd_result_get_device (res);
 	char *error_msg = NULL;
 	char *statement;
 	gboolean ret = TRUE;
@@ -426,11 +424,11 @@ fu_pending_set_state (FuPending *pending,
 	}
 
 	g_debug ("FuPending: set state of %s to %s",
-		 fwupd_result_get_device_id (res),
+		 fwupd_device_get_id (dev),
 		 fwupd_update_state_to_string (state));
 	statement = sqlite3_mprintf ("UPDATE pending SET state='%i' WHERE "
 				     "device_id = '%q';",
-				     state, fwupd_result_get_device_id (res));
+				     state, fwupd_device_get_id (dev));
 
 	/* remove entry */
 	rc = sqlite3_exec (priv->db, statement, NULL, NULL, &error_msg);
@@ -449,9 +447,6 @@ out:
 	return ret;
 }
 
-/**
- * fu_pending_set_error_msg:
- **/
 gboolean
 fu_pending_set_error_msg (FuPending *pending,
 			  FwupdResult *res,
@@ -459,6 +454,7 @@ fu_pending_set_error_msg (FuPending *pending,
 			  GError **error)
 {
 	FuPendingPrivate *priv = GET_PRIVATE (pending);
+	FwupdDevice *dev = fwupd_result_get_device (res);
 	char *error_msg = NULL;
 	char *statement;
 	gboolean ret = TRUE;
@@ -473,11 +469,11 @@ fu_pending_set_error_msg (FuPending *pending,
 	}
 
 	g_debug ("FuPending: add comment to %s: %s",
-		 fwupd_result_get_device_id (res), error_msg2);
+		 fwupd_device_get_id (dev), error_msg2);
 	statement = sqlite3_mprintf ("UPDATE pending SET error='%q' WHERE "
 				     "device_id = '%q';",
 				     error_msg2,
-				     fwupd_result_get_device_id (res));
+				     fwupd_device_get_id (dev));
 
 	/* remove entry */
 	rc = sqlite3_exec (priv->db, statement, NULL, NULL, &error_msg);
@@ -496,9 +492,6 @@ out:
 	return ret;
 }
 
-/**
- * fu_pending_class_init:
- **/
 static void
 fu_pending_class_init (FuPendingClass *klass)
 {
@@ -506,17 +499,11 @@ fu_pending_class_init (FuPendingClass *klass)
 	object_class->finalize = fu_pending_finalize;
 }
 
-/**
- * fu_pending_init:
- **/
 static void
 fu_pending_init (FuPending *pending)
 {
 }
 
-/**
- * fu_pending_finalize:
- **/
 static void
 fu_pending_finalize (GObject *object)
 {
@@ -529,9 +516,6 @@ fu_pending_finalize (GObject *object)
 	G_OBJECT_CLASS (fu_pending_parent_class)->finalize (object);
 }
 
-/**
- * fu_pending_new:
- **/
 FuPending *
 fu_pending_new (void)
 {
