@@ -27,7 +27,9 @@
 #include <glib-object.h>
 #include <gusb.h>
 
+#include "fu-common.h"
 #include "fu-device.h"
+#include "fu-hwids.h"
 
 G_BEGIN_DECLS
 
@@ -49,13 +51,14 @@ struct _FuPluginClass
 	void		 (* recoldplug)			(FuPlugin	*plugin);
 	void		 (* set_coldplug_delay)		(FuPlugin	*plugin,
 							 guint		 duration);
+	void		 (* device_register)		(FuPlugin	*plugin,
+							 FuDevice	*device);
 	/*< private >*/
-	gpointer	padding[25];
+	gpointer	padding[24];
 };
 
 typedef enum {
 	FU_PLUGIN_VERIFY_FLAG_NONE		= 0,
-	FU_PLUGIN_VERIFY_FLAG_USE_SHA256	= 1 << 0,
 	FU_PLUGIN_VERIFY_FLAG_LAST
 } FuPluginVerifyFlags;
 
@@ -76,6 +79,8 @@ void		 fu_plugin_device_add_delay		(FuPlugin	*plugin,
 							 FuDevice	*device);
 void		 fu_plugin_device_remove		(FuPlugin	*plugin,
 							 FuDevice	*device);
+void		 fu_plugin_device_register		(FuPlugin	*plugin,
+							 FuDevice	*device);
 void		 fu_plugin_set_status			(FuPlugin	*plugin,
 							 FwupdStatus	 status);
 void		 fu_plugin_set_percentage		(FuPlugin	*plugin,
@@ -84,7 +89,6 @@ void		 fu_plugin_recoldplug			(FuPlugin	*plugin);
 void		 fu_plugin_set_coldplug_delay		(FuPlugin	*plugin,
 							 guint		 duration);
 gboolean	 fu_plugin_has_device_delay		(FuPlugin	*plugin);
-GChecksumType	 fu_plugin_get_checksum_type		(FuPluginVerifyFlags flags);
 gpointer	 fu_plugin_cache_lookup			(FuPlugin	*plugin,
 							 const gchar	*id);
 void		 fu_plugin_cache_remove			(FuPlugin	*plugin,
@@ -92,6 +96,10 @@ void		 fu_plugin_cache_remove			(FuPlugin	*plugin,
 void		 fu_plugin_cache_add			(FuPlugin	*plugin,
 							 const gchar	*id,
 							 gpointer	 dev);
+gboolean	 fu_plugin_check_hwid			(FuPlugin	*plugin,
+							 const gchar	*hwid);
+const gchar	*fu_plugin_get_dmi_value		(FuPlugin	*plugin,
+							 const gchar	*dmi_id);
 
 G_END_DECLS
 
